<?php

declare(strict_types=1);

namespace Drupal\email_hidden_formatter\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Controller for the Email Hidden Formatter module.
 */
class EmailHiddenController extends ControllerBase {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Constructs a new EmailHiddenController object.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct($renderer) {
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create($container): static {
    return new static(
      $container->get('renderer')
    );
  }

  /**
   * Builds an AJAX response to dynamically render the email field.
   *
   * @param string $entity_type
   *   The entity type (e.g., 'node').
   * @param int $entity_id
   *   The ID of the entity containing the email field.
   * @param string $field_name
   *   The machine name of the field to render.
   * @param string $label
   *   The label visibility setting ('hidden', 'inline', 'above').
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The AJAX response object containing the rendered email field or an error message.
   */
  public function process(string $entity_type, int $entity_id, string $field_name, string $label): JsonResponse {
    // Load the storage for the specified entity type.
    $storage = $this->entityTypeManager()->getStorage($entity_type);
    if (!$storage) {
      return new JsonResponse(['error' => 'Invalid entity type.'], 400);
    }

    // Load the entity by ID.
    $entity = $storage->load($entity_id);
    if (!$entity instanceof EntityInterface) {
      return new JsonResponse(['error' => 'Entity not found.'], 404);
    }

    // Ensure the field exists on the entity.
    if (!$entity->hasField($field_name)) {
      return new JsonResponse(['error' => 'Field not found on this entity.'], 400);
    }

    // Ensure the field is not empty.
    $field = $entity->get($field_name);
    if ($field->isEmpty()) {
      return new JsonResponse(['error' => 'Field is empty.'], 400);
    }

    // Render the field with the specified label visibility.
    $render_array = $field->view([
      'label' => $label,
    ]);
    $rendered_field = $this->renderer->render($render_array);

    // Prepare the CSS-safe field name for the ReplaceCommand.
    $css_safe_field_name = str_replace('_', '-', $field_name);

    // Build and return the AJAX response.
    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand('.field--name-' . $css_safe_field_name, $rendered_field));

    return $response;
  }

}
